@@ -1,5 +1,5 @@
-/* $Id: cql.h,v 1.10 2004/11/01 20:13:02 adam Exp $
-   Copyright (C) 2002-2004
+/* $Id: cql.h,v 1.13 2005/06/27 22:03:59 adam Exp $
+   Copyright (C) 1995-2005, Index Data ApS
    Index Data Aps
 
 This file is part of the YAZ toolkit.
@@ -81,30 +81,30 @@ struct cql_node {
     /** node type */
     int which;
     union {
-	/** which == CQL_NODE_ST */
+        /** which == CQL_NODE_ST */
         struct {
-	    /** CQL index */
+            /** CQL index */
             char *index;
-	    /** CQL index URI or NULL if no URI */
-	    char *index_uri;
-	    /** Search term */
+            /** CQL index URI or NULL if no URI */
+            char *index_uri;
+            /** Search term */
             char *term;
-	    /** relation */
+            /** relation */
             char *relation;
-	    /** relation URL or NULL if no relation URI) */
-	    char *relation_uri;
-	    /** relation modifiers */
+            /** relation URL or NULL if no relation URI) */
+            char *relation_uri;
+            /** relation modifiers */
             struct cql_node *modifiers;
         } st;
-	/** which == CQL_NODE_BOOL */
+        /** which == CQL_NODE_BOOL */
         struct {
             /** operator name "and", "or", ... */
             char *value;
-	    /** left operand */
+            /** left operand */
             struct cql_node *left;
-	    /** right operand */ 
+            /** right operand */ 
             struct cql_node *right;
-	    /** modifiers (NULL for no list) */
+            /** modifiers (NULL for no list) */
             struct cql_node *modifiers;
         } boolean;
     } u;
@@ -153,7 +153,7 @@ struct cql_node *cql_node_mk_sc(NMEM nmem, const char *index,
  */
 YAZ_EXPORT
 struct cql_node *cql_apply_prefix(NMEM nmem, struct cql_node *cn,
-				  const char *prefix, const char *uri);
+                                  const char *prefix, const char *uri);
 
 /**
  * This function creates a boolean node.
@@ -281,7 +281,29 @@ const char *cql_strerror(int code);
 YAZ_EXPORT
 const char *cql_uri();
 
+/**
+ * Compares two CQL strings (for relations, operators, etc)
+ * (unfortunately defined as case-insensitive unlike XML etc)
+ */
+YAZ_EXPORT
+int cql_strcmp(const char *s1, const char *s2);
+
+/**
+ * Compares two CQL strings at most n bytes
+ * (unfortunately defined as case-insensitive unlike XML etc)
+ */
+YAZ_EXPORT
+int cql_strncmp(const char *s1, const char *s2, size_t n);
+
 YAZ_END_CDECL
 
 #endif
 /* CQL_H_INCLUDED */
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
