@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995-2004, Index Data.
+ * Copyright (C) 1995-2005, Index Data ApS
  *
  * Permission to use, copy, modify, distribute, and sell this software and
  * its documentation, in whole or in part, for any purpose, is hereby granted,
@@ -23,7 +23,7 @@
  * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
  * OF THIS SOFTWARE.
  *
- * $Id: pquery.h,v 1.5 2004/10/15 00:18:59 adam Exp $
+ * $Id: pquery.h,v 1.8 2006/04/01 11:45:23 adam Exp $
  */
 /**
  * \file pquery.h
@@ -78,6 +78,20 @@ YAZ_EXPORT int yaz_pqf_error (YAZ_PQF_Parser p, const char **msg, size_t *off);
 /* internal failure */
 #define YAZ_PQF_ERROR_INTERNAL 6
 
+/* bad proximity args */
+#define YAZ_PQF_ERROR_PROXIMITY 7
+
+/* bad unsigned integer */
+#define YAZ_PQF_ERROR_BAD_INTEGER 8
+
 YAZ_END_CDECL
 
 #endif
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
