@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 1998-2004, Index Data
+ * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: zgdu.h,v 1.3 2004/10/15 00:18:59 adam Exp $
+ * $Id: zgdu.h,v 1.6 2006/03/01 23:24:25 adam Exp $
  */
 
 /**
@@ -56,6 +56,10 @@ typedef struct {
 YAZ_EXPORT int z_GDU (ODR o, Z_GDU **p, int opt, const char *name);
 YAZ_EXPORT void z_HTTP_header_add(ODR o, Z_HTTP_Header **hp, const char *n,
                                   const char *v);
+YAZ_EXPORT void z_HTTP_header_add_content_type(ODR o, Z_HTTP_Header **hp,
+                                               const char *content_type,
+                                               const char *charset);
+
 YAZ_EXPORT const char *z_HTTP_header_lookup(Z_HTTP_Header *hp, const char *n);
 
 YAZ_EXPORT const char *z_HTTP_errmsg(int code);
@@ -66,3 +70,11 @@ YAZ_EXPORT Z_GDU *z_get_HTTP_Request(ODR o);
 YAZ_END_CDECL
 
 #endif
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
