@@ -2,7 +2,7 @@
  * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: zoom.h,v 1.25 2005/01/15 19:47:10 adam Exp $
+ * $Id: zoom.h,v 1.34 2006/04/07 11:27:24 adam Exp $
  */
 /**
  * \file zoom.h
@@ -29,8 +29,7 @@ ZOOM_BEGIN_CDECL
 typedef struct ZOOM_options_p *ZOOM_options;
 typedef struct ZOOM_query_p *ZOOM_query;
 typedef struct ZOOM_connection_p *ZOOM_connection;
-typedef	struct ZOOM_resultset_p *ZOOM_resultset;
-typedef struct ZOOM_task_p *ZOOM_task;
+typedef struct ZOOM_resultset_p *ZOOM_resultset;
 typedef struct ZOOM_record_p *ZOOM_record;
 typedef struct ZOOM_scanset_p *ZOOM_scanset;
 typedef struct ZOOM_package_p *ZOOM_package;
@@ -52,7 +51,7 @@ ZOOM_connection_create (ZOOM_options options);
 /* connect given existing connection */
 ZOOM_API(void)
 ZOOM_connection_connect(ZOOM_connection c, const char *host,
-			      int portnum);
+                              int portnum);
 
 /* destroy connection (close connection also) */
 ZOOM_API(void)
@@ -62,20 +61,23 @@ ZOOM_connection_destroy (ZOOM_connection c);
 ZOOM_API(const char *)
 ZOOM_connection_option_get (ZOOM_connection c, const char *key);
 
+ZOOM_API(const char *)
+ZOOM_connection_option_getl (ZOOM_connection c, const char *key, int *lenp);
+
 ZOOM_API(void)
 ZOOM_connection_option_set (ZOOM_connection c, const char *key,
                             const char *val);
 
 ZOOM_API(void)
 ZOOM_connection_option_setl (ZOOM_connection c, const char *key,
-			     const char *val, int len);
+                             const char *val, int len);
 
 /* return error code (0 == success, failure otherwise). cp
    holds error string on failure, addinfo holds addititional info (if any)
 */
 ZOOM_API(int)
 ZOOM_connection_error (ZOOM_connection c, const char **cp,
-			    const char **addinfo);
+                            const char **addinfo);
 
 ZOOM_API(int)
 ZOOM_connection_error_x (ZOOM_connection c, const char **cp,
@@ -90,6 +92,9 @@ ZOOM_connection_errmsg (ZOOM_connection c);
 /* returns additional info */
 ZOOM_API(const char *)
 ZOOM_connection_addinfo (ZOOM_connection c);
+/* returns diagnostic set */
+ZOOM_API(const char *)
+ZOOM_connection_diagset (ZOOM_connection c);
 /* translates error code into human-readable string */
 ZOOM_API(const char *)
 ZOOM_diag_str (int error);
@@ -106,6 +111,8 @@ ZOOM_diag_str (int error);
 #define ZOOM_ERROR_UNSUPPORTED_PROTOCOL 10008
 #define ZOOM_ERROR_UNSUPPORTED_QUERY 10009
 #define ZOOM_ERROR_INVALID_QUERY 10010
+#define ZOOM_ERROR_CQL_PARSE 10011
+#define ZOOM_ERROR_CQL_TRANSFORM 10012
 
 ZOOM_API(int)
 ZOOM_connection_last_event(ZOOM_connection cs);
@@ -120,6 +127,7 @@ ZOOM_connection_last_event(ZOOM_connection cs);
 #define ZOOM_EVENT_RECV_APDU 7
 #define ZOOM_EVENT_RECV_RECORD 8
 #define ZOOM_EVENT_RECV_SEARCH 9
+#define ZOOM_EVENT_END 10
 
 /* ----------------------------------------------------------- */
 /* result sets */
@@ -189,6 +197,9 @@ ZOOM_query_destroy(ZOOM_query s);
 /* CQL */
 ZOOM_API(int)
 ZOOM_query_cql(ZOOM_query s, const char *str);
+/* CQL translated client-side into RPN: `conn' is optional for diagnostics */
+ZOOM_API(int)
+ZOOM_query_cql2rpn(ZOOM_query s, const char *str, ZOOM_connection conn);
 /* PQF */
 ZOOM_API(int)
 ZOOM_query_prefix(ZOOM_query s, const char *str);
@@ -201,13 +212,16 @@ ZOOM_query_sortby(ZOOM_query s, const char *criteria);
 ZOOM_API(ZOOM_scanset)
 ZOOM_connection_scan (ZOOM_connection c, const char *startterm);
 
+ZOOM_API(ZOOM_scanset)
+ZOOM_connection_scan1 (ZOOM_connection c, ZOOM_query startterm);
+
 ZOOM_API(const char *)
 ZOOM_scanset_term(ZOOM_scanset scan, size_t pos,
                   int *occ, int *len);
 
 ZOOM_API(const char *)
 ZOOM_scanset_display_term(ZOOM_scanset scan, size_t pos,
-			  int *occ, int *len);
+                          int *occ, int *len);
 
 ZOOM_API(size_t)
 ZOOM_scanset_size(ZOOM_scanset scan);
@@ -241,10 +255,13 @@ ZOOM_API(void)
                              const char *val);
 
 /* ----------------------------------------------------------- */
-/* Sort */
+/* Sort.  First function is deprecated, use second instead */
 ZOOM_API(void)
     ZOOM_resultset_sort(ZOOM_resultset r,
-			const char *sort_type, const char *sort_spec);
+                        const char *sort_type, const char *sort_spec);
+ZOOM_API(int)
+    ZOOM_resultset_sort1(ZOOM_resultset r,
+                        const char *sort_type, const char *sort_spec);
 
 /* ----------------------------------------------------------- */
 /* options */
@@ -266,6 +283,9 @@ ZOOM_API(ZOOM_options)
 ZOOM_API(const char *)
 ZOOM_options_get (ZOOM_options opt, const char *name);
 
+ZOOM_API(const char *)
+ZOOM_options_getl (ZOOM_options opt, const char *name, int *lenp);
+
 ZOOM_API(void)
 ZOOM_options_set (ZOOM_options opt, const char *name, const char *v);
 
@@ -295,3 +315,11 @@ ZOOM_API(int)
 ZOOM_event (int no, ZOOM_connection *cs);
 
 ZOOM_END_CDECL
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
