@@ -2,7 +2,7 @@
  * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: ber_bool.c,v 1.3 2005/01/15 19:47:11 adam Exp $
+ * $Id: ber_bool.c,v 1.5 2005/08/11 14:21:55 adam Exp $
  */
 
 /** 
@@ -31,9 +31,6 @@ int ber_boolean(ODR o, int *val)
             return 0;
         if (odr_putc(o, *val) < 0)
             return 0;
-#ifdef ODR_DEBUG
-        fprintf(stderr, "[val=%d]\n", *val);
-#endif
         return 1;
     case ODR_DECODE:
         if ((res = ber_declen(o->bp, &len, odr_max(o))) < 0)
@@ -49,12 +46,17 @@ int ber_boolean(ODR o, int *val)
         }
         *val = *o->bp;
         o->bp++;
-#ifdef ODR_DEBUG
-        fprintf(stderr, "[val=%d]\n", *val);
-#endif
         return 1;
     case ODR_PRINT:
         return 1;
     default: odr_seterror(o, OOTHER, 11); return 0;
     }
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
