@@ -2,7 +2,7 @@
  * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: ber_null.c,v 1.3 2005/01/15 19:47:11 adam Exp $
+ * $Id: ber_null.c,v 1.5 2005/08/11 14:21:55 adam Exp $
  */
 
 /** 
@@ -28,9 +28,6 @@ int ber_null(ODR o)
     case ODR_ENCODE:
         if (odr_putc(o, 0X00) < 0)
             return 0;
-#ifdef ODR_DEBUG
-        fprintf(stderr, "[NULL]\n");
-#endif
         return 1;
     case ODR_DECODE:
         if (odr_max(o) < 1)
@@ -43,14 +40,19 @@ int ber_null(ODR o)
             odr_seterror(o, OPROTO, 12);
             return 0;
         }
-#ifdef ODR_DEBUG
-        fprintf(stderr, "[NULL]\n");
-#endif
         return 1;
     case ODR_PRINT:
-	return 1;
+        return 1;
     default:
-	odr_seterror(o, OOTHER, 13);
-	return 0;
+        odr_seterror(o, OOTHER, 13);
+        return 0;
     }
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
