@@ -1,5 +1,5 @@
-/* $Id: cqlstrer.c,v 1.3 2004/10/03 22:34:07 adam Exp $
-   Copyright (C) 2002-2004
+/* $Id: cqlstrer.c,v 1.6 2006/03/10 10:43:32 mike Exp $
+   Copyright (C) 1995-2005, Index Data ApS
    Index Data Aps
 
 This file is part of the YAZ toolkit.
@@ -17,7 +17,7 @@ See the file LICENSE details.
 /*
  * The error-messages associated with these codes are taken from
  * the SRW diagnostic specifications at
- *	http://www.loc.gov/srw/diagnostic-list.html
+ *      http://www.loc.gov/standards/sru/diagnostics-list.html
  */
 const char *cql_strerror(int code) {
     static char buf[80];
@@ -64,3 +64,11 @@ const char *cql_strerror(int code) {
     sprintf(buf, "Unknown CQL error #%d", code);
     return buf;
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
