@@ -2,7 +2,7 @@
  * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: odr_mem.c,v 1.4 2005/01/15 19:47:14 adam Exp $
+ * $Id: odr_mem.c,v 1.7 2006/02/19 18:33:09 adam Exp $
  */
 /**
  * \file odr_mem.c
@@ -32,14 +32,14 @@ NMEM odr_extract_mem(ODR o)
 void *odr_malloc(ODR o, int size)
 {
     if (o && !o->mem)
-	o->mem = nmem_create();
+        o->mem = nmem_create();
     return nmem_malloc(o ? o->mem : 0, size);
 }
 
 char *odr_strdup(ODR o, const char *str)
 {
     if (o && !o->mem)
-	o->mem = nmem_create();
+        o->mem = nmem_create();
     return nmem_strdup(o->mem, str);
 }
 
@@ -51,7 +51,7 @@ char *odr_strdupn(ODR o, const char *str, size_t n)
 int *odr_intdup(ODR o, int v)
 {
     if (o && !o->mem)
-	o->mem = nmem_create();
+        o->mem = nmem_create();
     return nmem_intdup(o->mem, v);
 }
 
@@ -60,6 +60,16 @@ int odr_total(ODR o)
     return o->mem ? nmem_total(o->mem) : 0;
 }
 
+Odr_oct *odr_create_Odr_oct(ODR o, const unsigned char *buf, int sz)
+{
+    Odr_oct *p = (Odr_oct *) odr_malloc(o, sizeof(Odr_oct));
+    p->buf = odr_malloc(o, sz);
+    memcpy(p->buf, buf, sz);
+    p->size = sz;
+    p->len = sz;
+    return p;
+}
+
 /* ---------- memory management for data encoding ----------*/
 
 
@@ -68,22 +78,19 @@ int odr_grow_block(ODR b, int min_bytes)
     int togrow;
 
     if (!b->can_grow)
-    	return -1;
+        return -1;
     if (!b->size)
-    	togrow = 1024;
+        togrow = 1024;
     else
-    	togrow = b->size;
+        togrow = b->size;
     if (togrow < min_bytes)
-    	togrow = min_bytes;
+        togrow = min_bytes;
     if (b->size && !(b->buf =
-		     (unsigned char *) xrealloc(b->buf, b->size += togrow)))
-    	abort();
+                     (unsigned char *) xrealloc(b->buf, b->size += togrow)))
+        abort();
     else if (!b->size && !(b->buf = (unsigned char *)
-			   xmalloc(b->size = togrow)))
-    	abort();
-#ifdef ODR_DEBUG
-    fprintf(stderr, "New size for encode_buffer: %d\n", b->size);
-#endif
+                           xmalloc(b->size = togrow)))
+        abort();
     return 0;
 }
 
@@ -92,26 +99,34 @@ int odr_write(ODR o, unsigned char *buf, int bytes)
     if (o->pos + bytes >= o->size && odr_grow_block(o, bytes))
     {
         odr_seterror(o, OSPACE, 40);
-	return -1;
+        return -1;
     }
     memcpy(o->buf + o->pos, buf, bytes);
     o->pos += bytes;
     if (o->pos > o->top)
-    	o->top = o->pos;
+        o->top = o->pos;
     return 0;
 }
 
 int odr_seek(ODR o, int whence, int offset)
 {
     if (whence == ODR_S_CUR)
-    	offset += o->pos;
+        offset += o->pos;
     else if (whence == ODR_S_END)
-    	offset += o->top;
+        offset += o->top;
     if (offset > o->size && odr_grow_block(o, offset - o->size))
     {
         odr_seterror(o, OSPACE, 41);
-	return -1;
+        return -1;
     }
     o->pos = offset;
     return 0;
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
