@@ -2,7 +2,7 @@
  * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: session.h,v 1.8 2005/03/08 11:07:49 adam Exp $
+ * $Id: session.h,v 1.10 2006/03/15 13:32:05 adam Exp $
  */
 /**
  * \file session.h
@@ -26,6 +26,8 @@ struct gfs_server {
     cql_transform_t cql_transform;
     void *server_node_ptr;
     char *directory;
+    char *docpath;
+    char *stylesheet;
     struct gfs_server *next;
 };
 
@@ -109,10 +111,12 @@ typedef struct association
     statserv_options_block *last_control;
     cql_transform_t cql_transform;
     void *server_node_ptr;
+    const char *docpath;
+    const char *stylesheet;
 } association;
 
 association *create_association(IOCHAN channel, COMSTACK link,
-				const char *apdufile);
+                                const char *apdufile);
 void destroy_association(association *h);
 void ir_session(IOCHAN h, int event);
 
@@ -130,3 +134,11 @@ int statserv_must_terminate(void);
 int control_association(association *assoc, const char *host, int force);
 
 #endif
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
