@@ -2,7 +2,7 @@
  * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: zoom-p.h,v 1.6 2005/01/15 19:47:15 adam Exp $
+ * $Id: zoom-p.h,v 1.10 2006/03/01 23:24:26 adam Exp $
  */
 /**
  * \file zoom-p.h
@@ -24,6 +24,16 @@ struct ZOOM_query_p {
     char *query_string;
 };
 
+typedef enum {
+    zoom_sru_error,
+    zoom_sru_soap,
+    zoom_sru_get,
+    zoom_sru_post,
+} zoom_sru_mode;
+    
+
+typedef struct ZOOM_task_p *ZOOM_task;
+
 #define STATE_IDLE 0
 #define STATE_CONNECTING 1
 #define STATE_ESTABLISHED 2
@@ -63,11 +73,13 @@ struct ZOOM_connection_p {
     ZOOM_resultset resultsets;
     ZOOM_Event m_queue_front;
     ZOOM_Event m_queue_back;
+    zoom_sru_mode sru_mode;
 };
 
 struct ZOOM_options_entry {
     char *name;
     char *value;
+    int len;                  /* of `value', which may contain NULs */
     struct ZOOM_options_entry *next;
 };
 
@@ -158,10 +170,10 @@ struct ZOOM_task_p {
 #define ZOOM_TASK_PACKAGE 5
         ZOOM_package package;
 #define ZOOM_TASK_SORT 6
-	struct {
-	    ZOOM_resultset resultset;
-	    ZOOM_query q;
-	} sort;
+        struct {
+            ZOOM_resultset resultset;
+            ZOOM_query q;
+        } sort;
     } u;
     ZOOM_task next;
 };
@@ -173,3 +185,11 @@ struct ZOOM_Event_p {
 };
 
 void ZOOM_options_addref (ZOOM_options opt);
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
