@@ -2,14 +2,14 @@
  * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: tstccl.c,v 1.8 2005/04/15 21:47:56 adam Exp $
+ * $Id: tstccl.c,v 1.10 2006/01/29 21:59:13 adam Exp $
  */
 
 /* CCL test */
 
-#include <stdlib.h>
 #include <string.h>
 #include <yaz/ccl.h>
+#include <yaz/test.h>
 
 struct ccl_tst {
     char *query;
@@ -44,13 +44,15 @@ static struct ccl_tst query_str[] = {
     {0, 0}
 };
 
-void tst1(int pass, int *number_of_errors)
+void tst1(int pass)
 {
     CCL_parser parser = ccl_parser_create ();
     CCL_bibset bibset = ccl_qual_mk();
     int i;
     char tstline[128];
 
+    YAZ_CHECK(parser);
+    YAZ_CHECK(bibset);
     switch(pass)
     {
     case 0:
@@ -59,9 +61,9 @@ void tst1(int pass, int *number_of_errors)
         ccl_qual_fitem(bibset, "1=/my/title",         "dc.title");
         ccl_qual_fitem(bibset, "r=r",         "date");
         ccl_qual_fitem(bibset, "r=o",         "x");
-	break;
+        break;
     case 1:
-	strcpy(tstline, "ti u=4    s=pw t=l,r");
+        strcpy(tstline, "ti u=4    s=pw t=l,r");
         ccl_qual_line(bibset, tstline);
 
         strcpy(tstline, "term 1=1016 s=al,pw   # default term");
@@ -75,75 +77,68 @@ void tst1(int pass, int *number_of_errors)
 
         strcpy(tstline, "x r=o # ordered relation");
         ccl_qual_line(bibset, tstline);
-	break;
+        break;
     case 2:
         ccl_qual_buf(bibset, "ti u=4    s=pw t=l,r\n"
-		     "term 1=1016 s=al,pw\r\n"
-		     "\n"
-		     "dc.title 1=/my/title\n"
-		     "date r=r\n" 
-		     "x r=o\n"
-	    );
-	break;
+                     "term 1=1016 s=al,pw\r\n"
+                     "\n"
+                     "dc.title 1=/my/title\n"
+                     "date r=r\n" 
+                     "x r=o\n"
+            );
+        break;
     default:
-	exit(23);
+        YAZ_CHECK(0);
     }
 
     parser->bibset = bibset;
 
     for (i = 0; query_str[i].query; i++)
     {
-	struct ccl_token *token_list;
-	struct ccl_rpn_node *rpn;
+        struct ccl_token *token_list;
+        struct ccl_rpn_node *rpn;
 
-	token_list = ccl_parser_tokenize(parser, query_str[i].query);
-	rpn = ccl_parser_find(parser, token_list);
-	ccl_token_del (token_list);
-	if (rpn)
-	{
-	    WRBUF wrbuf = wrbuf_alloc();
-	    ccl_pquery(wrbuf, rpn);
+        token_list = ccl_parser_tokenize(parser, query_str[i].query);
+        rpn = ccl_parser_find(parser, token_list);
+        ccl_token_del (token_list);
+        if (rpn)
+        {
+            /* parse ok. check that result is there and match */
+            WRBUF wrbuf = wrbuf_alloc();
+            ccl_pquery(wrbuf, rpn);
 
-	    if (!query_str[i].result)
-	    {
-		printf ("Failed %s\n", query_str[i].query);
-		printf (" got:     %s:\n", wrbuf_buf(wrbuf));
-		printf (" expected failure\n");
-		(*number_of_errors)++;
-	    }
-	    else if (strcmp(wrbuf_buf(wrbuf), query_str[i].result))
-	    {
-		printf ("Failed %s\n", query_str[i].query);
-		printf (" got:     %s:\n", wrbuf_buf(wrbuf));
-		printf (" expected:%s:\n", query_str[i].result);
-		(*number_of_errors)++;
-	    }
-	    ccl_rpn_delete(rpn);
-	    wrbuf_free(wrbuf, 1);
-	}
-	else if (query_str[i].result)
-	{
-	    printf ("Failed %s\n", query_str[i].query);
-	    printf (" got failure\n");
-	    printf (" expected:%s:\n", query_str[i].result);
-	    (*number_of_errors)++;
-	}
-    }	
+            /* check expect a result and that it matches */
+            YAZ_CHECK(query_str[i].result);
+            if (query_str[i].result)
+            {
+                YAZ_CHECK(!strcmp(wrbuf_buf(wrbuf), query_str[i].result));
+            }
+            ccl_rpn_delete(rpn);
+            wrbuf_free(wrbuf, 1);
+        }
+        else 
+        {
+            /* parse failed. So we expect no result either */
+            YAZ_CHECK(!query_str[i].result);
+        }
+    }   
     ccl_parser_destroy (parser);
     ccl_qual_rm(&bibset);
 }
 
 int main(int argc, char **argv)
 {
-    int number_of_errors = 0;
-    tst1(0, &number_of_errors);
-    if (number_of_errors)
-	exit(1);
-    tst1(1, &number_of_errors);
-    if (number_of_errors)
-	exit(1);
-    tst1(2, &number_of_errors);
-    if (number_of_errors)
-	exit(1);
-    exit(0);
+    YAZ_CHECK_INIT(argc, argv);
+    tst1(0);
+    tst1(1);
+    tst1(2);
+    YAZ_CHECK_TERM;
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
