@@ -1,14 +1,15 @@
 /*
- * Copyright (c) 1997-2003, Index Data
+ * Copyright (C) 1995-2005, Index Data ApS
  * See the file LICENSE for details.
  *
- * $Id: yaziconv.c,v 1.1 2003/04/23 20:34:08 adam Exp $
+ * $Id: yaziconv.c,v 1.5 2006/04/21 10:28:07 adam Exp $
  */
 
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <errno.h>
 #include <string.h>
 #include <ctype.h>
@@ -50,8 +51,8 @@ void convert (FILE *inf, yaz_iconv_t cd, int verbose)
         }
         if (verbose > 1)
         {
-            fprintf (stderr, "yaz_iconv: inbytesleft=%d outbytesleft=%d\n",
-                     inbytesleft, outbytesleft);
+            fprintf (stderr, "yaz_iconv: inbytesleft=%ld outbytesleft=%ld\n",
+                     (long) inbytesleft, (long) outbytesleft);
 
         }
         r = yaz_iconv (cd, &inbuf, &inbytesleft, &outbuf, &outbytesleft);
@@ -179,3 +180,11 @@ int main (int argc, char **argv)
     yaz_iconv_close (cd);
     return 0;
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
