@@ -1,9 +1,11 @@
-/*
- * $Id: zoomtst4.c,v 1.7 2001/11/30 08:24:06 adam Exp $
- *
- * Asynchronous multi-target going through proxy doing search and retrieve
- * using present.
- */
+/* $Id: zoomtst4.c,v 1.9 2006/04/21 10:28:08 adam Exp $  */
+
+/** \file zoomtst4.c
+    \brief Asynchronous multi-target client with separate present
+    
+    Asynchronous multi-target going through proxy doing search and retrieve
+    using present.
+*/
 
 #include <stdio.h>
 #include <string.h>
@@ -14,7 +16,7 @@
 const char *my_callback (void *handle, const char *name)
 {
     if (!strcmp (name, "async"))
-	return "1";
+        return "1";
     return 0;
 }
 
@@ -29,9 +31,9 @@ int main(int argc, char **argv)
 
     if (argc < 4)
     {
-	fprintf (stderr, "usage:\n%s proxy target1 .. targetN query\n",
-		 *argv);
-	exit (2);
+        fprintf (stderr, "usage:\n%s proxy target1 .. targetN query\n",
+                 *argv);
+        exit (2);
     }
     if (no > 500)
         no = 500;
@@ -52,33 +54,33 @@ int main(int argc, char **argv)
     q = ZOOM_query_create ();
     if (ZOOM_query_prefix (q, argv[argc-1]))
     {
-	printf ("bad PQF: %s\n", argv[argc-1]);
-	exit (1);
+        printf ("bad PQF: %s\n", argv[argc-1]);
+        exit (1);
     }
     /* connect - and search all */
     for (i = 0; i<no; i++)
     {
-    	z[i] = ZOOM_connection_create (o);
-    	ZOOM_connection_connect (z[i], argv[i+2], 0);
+        z[i] = ZOOM_connection_create (o);
+        ZOOM_connection_connect (z[i], argv[i+2], 0);
         r[i] = ZOOM_connection_search (z[i], q);
     }
 
     /* network I/O */
     while (ZOOM_event (no, z))
-	;
+        ;
 
     /* handle errors */
     for (i = 0; i<no; i++)
     {
-	int error;
-	const char *errmsg, *addinfo;
-	if ((error = ZOOM_connection_error(z[i], &errmsg, &addinfo)))
-	    fprintf (stderr, "%s error: %s (%d) %s\n",
-		     ZOOM_connection_option_get(z[i], "host"),
+        int error;
+        const char *errmsg, *addinfo;
+        if ((error = ZOOM_connection_error(z[i], &errmsg, &addinfo)))
+            fprintf (stderr, "%s error: %s (%d) %s\n",
+                     ZOOM_connection_option_get(z[i], "host"),
                      errmsg, error, addinfo);
         else
-	    printf ("%s: %d hits\n", ZOOM_connection_option_get(z[i], "host"),
-                    ZOOM_resultset_size(r[i]));
+            printf ("%s: %ld hits\n", ZOOM_connection_option_get(z[i], "host"),
+                    (long) ZOOM_resultset_size(r[i]));
     }
 
     /* destroy stuff and exit */
@@ -91,3 +93,11 @@ int main(int argc, char **argv)
     ZOOM_options_destroy(o);
     exit (0);
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
