@@ -1,8 +1,8 @@
-/*
- * $Id: zoomtst8.c,v 1.3 2002/05/21 08:36:04 adam Exp $
- *
- * Asynchronous multi-target client doing scan
- */
+/* $Id: zoomtst8.c,v 1.5 2006/04/21 10:28:08 adam Exp $  */
+
+/** \file zoomtst8.c
+    \brief Asynchronous multi-target client doing scan
+*/
 
 #include <stdio.h>
 #include <string.h>
@@ -21,9 +21,9 @@ int main(int argc, char **argv)
 
     if (argc < 3)
     {
-	fprintf (stderr, "usage:\n%s target1 target2 ... targetN scan\n",
-		 *argv);
-	exit (1);
+        fprintf (stderr, "usage:\n%s target1 target2 ... targetN scan\n",
+                 *argv);
+        exit (1);
     }
     if (no > 500)
         no = 500;
@@ -34,11 +34,11 @@ int main(int argc, char **argv)
     /* connect to all */
     for (i = 0; i<no; i++)
     {
-	/* create connection - pass options (they are the same for all) */
-    	z[i] = ZOOM_connection_create (o);
+        /* create connection - pass options (they are the same for all) */
+        z[i] = ZOOM_connection_create (o);
 
-	/* connect and init */
-    	ZOOM_connection_connect (z[i], argv[1+i], 0);
+        /* connect and init */
+        ZOOM_connection_connect (z[i], argv[1+i], 0);
         
     }
     /* scan all */
@@ -56,15 +56,15 @@ int main(int argc, char **argv)
 
     for (i = 0; i<no; i++)
     {
-	int error;
-	const char *errmsg, *addinfo;
-	if ((error = ZOOM_connection_error(z[i], &errmsg, &addinfo)))
-	    fprintf (stderr, "%s error: %s (%d) %s\n",
-		     ZOOM_connection_option_get(z[i], "host"),
+        int error;
+        const char *errmsg, *addinfo;
+        if ((error = ZOOM_connection_error(z[i], &errmsg, &addinfo)))
+            fprintf (stderr, "%s error: %s (%d) %s\n",
+                     ZOOM_connection_option_get(z[i], "host"),
                      errmsg, error, addinfo);
         else
         {
-            size_t j;
+            int j;
             printf ("%s\n", ZOOM_connection_option_get(z[i], "host"));
             for (j = 0; j<ZOOM_scanset_size (s[i]); j++)
             {
@@ -86,3 +86,11 @@ int main(int argc, char **argv)
     ZOOM_options_destroy(o);
     exit (0);
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
